<?php
/*
Plugin Name: PictPress
Plugin URI: http://www.curioso.org/pictpress/
Description: Adds fotolog functionality to wordpress posts by adding thumbnails and picture pages automatically to the post for all pictures stored at a post specific location. To install or upgrade <a href="../wp-content/plugins/pictpress/upgrade.php">press here</a> and then enable the plugin. For additional information see the <a href="../wp-content/plugins/pictpress/readme.html">readme file</a>.
Version: 0.99d
Author: Curioso
Author URI: http://www.curioso.org/
*/ 

// Location of PictPress directory relative to WordPress directory
//
$pp_pictpress_dir = "wp-content/plugins/pictpress";

require_once(ABSPATH . "$pp_pictpress_dir/wp13.php");

// Patterns for thumbnails and picture pages
//
$pp_thumbs_pat = "/<!--pp-thumb-start-->.*<!--pp-thumb-end-->/s";
$pp_pict_pat = "/<!--pp-page-start-->.*<!--pp-page-end-->/s";
$pp_comment_pat = "/<!--PictPress generated page for image (.*?)-->/s";

// Div to clear floats
//
$pp_clear = "<div class=\"clear\">&nbsp;</div>";

//////////////////////
// Template functions
//////////////////////

// Generate thumbnail for previous page
//
function pp_prev_thumb() {
	global $wpdb, $posts, $pp_comment_pat, $pp_pictpress_dir, $pp_clear;
	if (get_settings('pp_single_post') || 
	    !is_single() || 
	    $posts[0]->post_parent == 0)
		return;
	$post = $posts[0];
        $current_post_date = $post->post_modified;
	$prev = $wpdb->get_row("SELECT ID, post_content FROM $wpdb->posts 
			        WHERE post_parent = $post->post_parent
			        AND post_modified < '$current_post_date' 
			        AND post_status = 'publish' 
			        ORDER BY post_modified DESC LIMIT 1");
	if ($prev && preg_match($pp_comment_pat, $prev->post_content, $m)) {
		include_once (ABSPATH . "$pp_pictpress_dir/class.image.php");
		$image = new Image($m[1], $prev->ID);
		$r = get_permalink($prev->ID);
		$t = pp_get_title($prev->ID);
		$pp = __('Previous picture');
		echo "<li id=\"thumbnail\"><a href=\"$r\" title=\"$t\">$pp</a>";
		echo pp_expand_subtitles($image->GetThumbnail($r));
		echo "$pp_clear</li>\n";
	}
}

// Generate thumbnail for next page
//
function pp_next_thumb() {
	global $wpdb, $posts, $pp_comment_pat, $pp_pictpress_dir, $pp_clear;
	if (get_settings('pp_single_post') || 
	    !is_single() || 
	    $posts[0]->post_parent == 0)
		return;
	$post = $posts[0];
        $current_post_date = $post->post_modified;
	$next = $wpdb->get_row("SELECT ID, post_content FROM $wpdb->posts 
			        WHERE post_parent = $post->post_parent
			        AND post_modified > '$current_post_date' 
			        AND post_status = 'publish' 
			        ORDER BY post_modified ASC LIMIT 1");
	if ($next && preg_match($pp_comment_pat, $next->post_content, $m)) {
		include_once (ABSPATH . "$pp_pictpress_dir/class.image.php");
		$image = new Image($m[1], $next->ID);
		$r = get_permalink($next->ID);
		$t = pp_get_title($next->ID);
		$np = __('Next picture');
		echo "<li id=\"thumbnail\"><a href=\"$r\" title=\"$t\">$np</a>";
		echo pp_expand_subtitles($image->GetThumbnail($r));
		echo "$pp_clear</li>\n";
	}
}

// Generate navigation thumbnails
//
function pp_navigation() {
	global $posts;
	if (!is_single() || 
	    $posts[0]->post_parent == 0)
		return;
	$href = get_permalink($posts[0]->post_parent);
	$title = pp_get_title($posts[0]->post_parent);
	$nav = __('Navigation');
	echo "<li id=\"navigation\">$nav<ul>";
	echo "<li><a href = \"$href\" title=\"$title\">$title</a></li>\n";
	pp_prev_thumb();
	pp_next_thumb();
	echo "</ul></li>\n";
}

// Print post last modified date/time
//
function pp_modified_date ($d='', $before='', $after='', $echo = true) {
        global $post;
        if ($d == "")
                $d = get_settings('date_format');
        $date = mysql2date($d, $post->post_date);
        $modified = mysql2date($d, $post->post_modified);
        $the_date = '';
        if ($date != $modified) {
                $the_date .= $before;
                $the_date .= mysql2date($d, $post->post_modified);
                $the_date .= $after;
        }
        $the_date = apply_filters('the_date', $the_date);
        if ($echo) {
                echo $the_date;
        } else {
                return $the_date;
        }
}

//////////////////////
// Internal functions
//////////////////////

// Get image directory
//
function pp_get_image_path($post_ID) {
	global $wpdb;
        $post = $wpdb->get_row("SELECT post_date, post_name 
                                FROM $wpdb->posts WHERE ID = $post_ID");

        $rewritecode = array(
                '%year%',
                '%monthnum%',
                '%day%',
                '%postname%',
                '%post_id%'
        );
	$unixtime = strtotime($post->post_date);
        $rewritereplace = array(
                date('Y', $unixtime),
                date('m', $unixtime),
                date('d', $unixtime),
                $post->post_name,
                $post_ID
        );
	$perm = str_replace($rewritecode, $rewritereplace, 
			    get_settings('pp_image_dir')); 
	return $perm;
}

// Get subpost with image
//
function pp_get_subpost($parent, $slug) {
	global $wpdb;

	$sql = "SELECT ID FROM $wpdb->posts
		WHERE post_parent = '$parent' AND post_name ='$slug'";
	return $wpdb->get_var($sql);
}

// Create picture post
//
function pp_make_picture_post ($postdata, $image, $timestamp) {
	global $pp_pict_pat, $wpdb;

	$name = basename($image->name);
	$slug = preg_replace('/(.+)\..*$/', '$1', $name);
	$slug = sanitize_title($slug);
	$post_ID = $postdata['ID'];
	$post_modified = gmdate('Y-m-d H:i:s', $timestamp + 3600 *
                                            get_settings('gmt_offset'));
	$post_modified_gmt = gmdate('Y-m-d H:i:s', $timestamp);

	// Check if there is an existing post with this image
	//
	$image->id = $id = pp_get_subpost($post_ID, $slug);
	if ($id) {
		$postdata = wp_get_single_post($id, ARRAY_A);
		$content = $postdata['content'];

		// If existing picture, replace it
		// else append to end of content
		//
		if (preg_match($pp_pict_pat, $content))
			$content = preg_replace($pp_pict_pat,
					$image->GetPicture(),
					$content);
		else
			$content .= $image->GetPicture();
		$content = $wpdb->escape($content);
		if (get_settings('pp_use_picture_time')) {
			$post_date = gmdate('Y-m-d H:i:s', 
					    $image->time + 3600 *
					    get_settings('gmt_offset'));
			$post_date_gmt = gmdate('Y-m-d H:i:s', $image->time);
		} else {
			$post_date = $postdata['post_date'];
			$post_date_gmt = $postdata['post_date_gmt'];
		}
		$result = $wpdb->query("
			UPDATE $wpdb->posts SET
				post_content = '$content',
				post_date = '$post_date',
				post_date_gmt = '$post_date_gmt',
				post_modified = '$post_modified',
				post_modified_gmt = '$post_modified_gmt'
			WHERE ID = $id");
	} else {
		// Create new post
		//
		$postdata['post_parent'] = $post_ID;
		$postdata['post_title'] = $slug;
		if (get_settings('pp_use_picture_time')) {
			$postdata['post_date'] = gmdate('Y-m-d H:i:s', 
						$image->time + 3600 *
						get_settings('gmt_offset'));
			$postdata['post_date_gmt'] = gmdate('Y-m-d H:i:s', 
							    $image->time);
		}
		$postdata['post_content'] = $image->GetPicture();
		$cat = get_settings('pp_image_cat');
		if ($cat)
			$postdata['post_category'] = array($cat);
		$image->id = $id = wp_insert_post($postdata);
		$title = $wpdb->escape($image->GetText('pp_image_title'));
		// Fill in parent, title and modified times
		//
		$result = $wpdb->query("
			UPDATE $wpdb->posts SET
				post_parent = $post_ID,
				post_title = '$title',
				post_modified = '$post_modified',
				post_modified_gmt = '$post_modified_gmt'
			WHERE ID = $id ");
	}
	return $id;
}

// Create picture page
//
function pp_make_picture_page ($pages, $image) {
	global $pp_pict_pat;

	$numpages = count($pages);
	$name = basename($image->name);

	// Check if there is an existing page with this image
	//
	$page = '';
	for ($p = 1; $p < $numpages; $p++) {
		if (preg_match("/<!--pp-page-start-->.*$name.*<!--pp-page-end-->/s", $pages[$p])) {
			$page = $pages[$p];
			break;
		}
	}
	if ($page) {
		// If existing page, replace picture
		//
		$page = preg_replace($pp_pict_pat,
				     $image->GetPicture(),
				     $page);
	} else {
		// If not, generate a picture page
		//
		$page  = "<h4 class=\"caption\">";
		$page .= $image->GetText('pp_image_title');
		$page .= "</h4>";
		$page .= $image->GetPicture();
	}
	return $page;
}

// Add pictures to post, to be called by do_action('publish_post',...)
//
function pp_add_pictures ($post_ID) {
        global $pp_pictpress_dir, $wpdb;
	global $pp_clear, $pp_pict_pat, $pp_thumbs_pat;

	$postdata = wp_get_single_post($post_ID, ARRAY_A);

	// No recursive posts
	//
	if ($postdata['post_parent'] != 0)
		return;

	include_once (ABSPATH . "$pp_pictpress_dir/class.image.php");
	include_once (ABSPATH . "$pp_pictpress_dir/class.imagedir.php");

	$path = pp_get_image_path($post_ID);
	$dirname = get_settings('fileupload_realpath') . "/$path";

        if (file_exists($dirname) && is_dir($dirname))
		$dir = new ImageDir($path);

	// Remove previous thumbnails
	//
	$content = preg_replace($pp_thumbs_pat, "", $postdata['post_content']);

	$pages = explode('<!--nextpage-->', $content);
	$numpages = count($pages);

	// Copy all pages that contain no PictPress images
	//
	for ($p = 0; $p < $numpages; $p++) {
		if ($p == 0 || !preg_match($pp_pict_pat, $pages[$p])) {
			$newpages[] = $pages[$p];
		}
	}

	// Determine number of first page after thumbnails
	//
	if ($dir) {
		$pp_max_thumbs = get_settings('pp_max_thumbs');
		if ($dir->count < get_settings('pp_min_thumbs')) {
			$no_thumbs = 1;
		} else if ($pp_max_thumbs) {
			$first = count($newpages) + ceil($dir->count / $pp_max_thumbs);
		} else {
			$first = count($newpages) + 1;
		}
	}

	// Delete children if necessary
	//
	if ($no_thumbs || get_settings('pp_single_post'))
		pp_delete_children($post_ID);
	else
		pp_check_deleted($post_ID);

	// Create thumbnails
	//
	$thumbs = "<!--pp-thumb-start-->";
	if (!$dir) {
		$thumbs .= "<!--PictPress found no dir $dirname-->";
	} else if ($dir->count == 0) {
		$thumbs .= "<!--PictPress found no images in dir $dirname-->";
	} else {
		$thumbs .= "<!--PictPress generated thumbnails for dir $dirname-->";
		$pp_more_thumbs    = get_settings('pp_more_thumbs');
		$pp_thumbs_per_row = get_settings('pp_thumbs_per_row');

		$thumbs .= "$pp_clear";
		$timestamp = time();
		for ($i = 0; $i < $dir->count; $i++) {
			if ($pp_more_thumbs && $i == $pp_more_thumbs) {
				$thumbs .= "<!--more-->";
			}
			if ($i && $pp_max_thumbs) {
				$j = $i % $pp_max_thumbs;
				if ($j == 0) {
					$thumbs .= "$pp_clear<!--nextpage-->$pp_clear";
				} else if ($pp_thumbs_per_row && 
					   ($j % $pp_thumbs_per_row) == 0) {
					$thumbs .= "$pp_clear";
				}
			} else if ($i && $pp_thumbs_per_row && 
				   ($i % $pp_thumbs_per_row) == 0) {
				$thumbs .= "$pp_clear";
			}
			$image = $dir->GetImage($i);

			if ($no_thumbs) {
				$thumbs .= $image->GetPicture();
			} else {
				if (get_settings('pp_single_post')) {
					$image->id = $first + $i;
					$newpages[] = 
						pp_make_picture_page($pages, 
								     $image);
					$href = pp_get_link($post_ID, 
							    $image->id);
				} else {
					$image->id = pp_make_picture_post(
							$postdata, 
						     	$image, 
							$timestamp + $i);
					$href = get_permalink($image->id);
				}
				$thumbs .= $image->GetThumbnail($href);
			}
		}
		$thumbs .= "$pp_clear";
	}
	$thumbs .= "<!--pp-thumb-end-->";

	// Append thumbnails to end of first page.
	//
	$newpages[0] .= $thumbs;
	$content = $wpdb->escape(implode("<!--nextpage-->", $newpages));

        // Update database
        $result = $wpdb->query("
                UPDATE $wpdb->posts SET
                        post_content = '$content'
                WHERE ID = $post_ID ");

	return $post_ID;
}

// Add CSS header (to be called from wp_head)
//
function pp_css_head ($arg) {
	global $pp_pictpress_dir;

	$site = get_settings('siteurl');
	echo "<style type=\"text/css\">";
	echo "@import url( $site/$pp_pictpress_dir/pictpress.css );";
        echo "</style>";

	return $arg;
}

// Get subtitle for page $p
//
function pp_get_subtitle ($p) {
	global $pages;

	if (preg_match("'<h4.*?>(.*?)</h4>'s", $pages[$p-1], $matches))
		return $matches[1];
	else
		return "Page $p";
}

// Get title for post $id
//
function pp_get_title ($id) {
	global $wpdb;

	$sql = "SELECT post_title FROM $wpdb->posts WHERE ID = '$id'";
	$sub = $wpdb->get_var($sql);
	return $sub ? $sub : $id;
}

// Get link for page $p
//
function pp_get_link ($post_ID, $p) {
	global $querystring_equal, $querystring_separator;

	if ('' == get_settings('permalink_structure')) {
		return get_permalink($post_ID).$querystring_separator.'page'.$querystring_equal.$p;
	} else {
		return get_permalink($post_ID).$p.'/';
	}

}

// Generate thumbnail for picture post
//
function pp_get_thumb ($url) {
	global $wpdb, $pp_comment_pat, $pp_pictpress_dir;
	$id = url_to_postid ($url);
	if (!$id)
		return "<!--PictPress could not find post '$url'-->";
	$postdata = wp_get_single_post($id, ARRAY_A);
	$parent = $postdata['post_parent'];
	// If main post, generate thumbnail for first child
	if ($parent == 0) {
		$post = $wpdb->get_row("SELECT ID, post_content 
					FROM $wpdb->posts 
					WHERE post_parent = $id
					AND post_status = 'publish' 
					ORDER BY post_modified ASC LIMIT 1");
		$content = $post->post_content;
	} else {
		$content = $postdata['post_content'];
	}
	if (preg_match($pp_comment_pat, $content, $m)) {
		include_once (ABSPATH . "$pp_pictpress_dir/class.image.php");
		$image = new Image($m[1], $id);
		$r = get_permalink($id);
		return pp_expand_subtitles($image->GetThumbnail($r));
	} else {
		return "<!--PictPress could not find picture in post '$id'-->";
	}
}

// Expand subtitles in content (to be called as filter in the_content)
//
function pp_expand_subtitles ($content) {
	$content = preg_replace_callback(
			"/<!--subtitle +([0-9]+)-->/", 
			create_function('$matches',
				'return pp_get_subtitle($matches[1]);'),
			$content);
	$content = preg_replace_callback(
		        "/<!--pp-title +(.+?)-->/",
		        create_function('$matches',
				'return pp_get_title($matches[1]);'),
			$content);
	$content = preg_replace_callback(
		        "/<!--pp-thumb +(.+?)-->/",
		        create_function('$matches',
				'return pp_get_thumb($matches[1]);'),
			$content);
	// This prevents some wpautop problems but not all...
	//
	//$content = preg_replace('/<!--.*?-->/', 
			    //'<div class="comment">$0</div>',
			    //$content);
	return $content;
}

// Delete all posts with parent $id
//
function pp_delete_children ($id) {
	global $wpdb;
	global $post_id;

	if (!$id) $id = $post_id; // Bug in post.php when calling do_action

	$children = $wpdb->get_col("SELECT ID FROM $wpdb->posts
		                    WHERE post_parent = $id");
	if ($children) {
		foreach ($children as $child) {
			wp_delete_post($child);
			pp_delete_children($child);
		}
	}

	return $id;
}

// Check children with deleted image and delete them
//
function pp_check_deleted ($id) {
	global $wpdb, $pp_comment_pat;

	$children = $wpdb->get_col("SELECT ID FROM $wpdb->posts
		                    WHERE post_parent = $id");
	if ($children) {
		$dir = get_settings('fileupload_realpath');
		foreach ($children as $child) {
			$content = $wpdb->get_var("SELECT post_content 
						   FROM $wpdb->posts 
						   WHERE ID = $child");
			if (preg_match($pp_comment_pat, $content, $m) &&
			    !file_exists("$dir/" . $m[1])) {
				wp_delete_post($child);
				pp_delete_children($child);
		}
	}
}

// Filter out all child posts
//
function pp_filter_children () {
	global $posts;

	$newsposts = array();

	if ($posts && !is_single()) {
		foreach ($posts as $post) 
			if ($post->post_parent == 0)
				$newposts[] = $post;

		if ($newposts)
			$posts = $newposts;
	}
}

// Filter to add parent to title
//
function pp_add_parent_title ($title) {
	global $posts, $wpdb;

	$post = $posts[0];
	while ($post->post_parent != 0) {
		$post = $wpdb->get_row("SELECT post_parent, post_title 
					FROM $wpdb->posts 
					WHERE ID = $post->post_parent");
		$title = "$post->post_title &raquo; $title";
	}
	return $title;
}
	
// Priority must be below generic_ping priority, because
// generic_ping does not return its argument, so we would
// not have the post_ID we need.
//
add_action("publish_post", "pp_add_pictures", 5);
add_action("delete_post", "pp_delete_children");
add_action("wp_head", "pp_filter_children");
add_filter("single_post_title", "pp_add_parent_title");
add_filter("the_content", "pp_expand_subtitles", 5);
add_filter("the_excerpt", "pp_expand_subtitles", 5);
if (get_settings('pp_use_css')) add_action("wp_head", "pp_css_head");

//echo "PictPress loaded\n";

?>
